import { TableOptions } from '@nocobase/database';
import { getInterfaceFields } from '../interfaces';

export default {
  name: 'fields',
  title: '字段配置',
  internal: true,
  model: 'FieldModel',
  developerMode: true,
  createdAt: false,
  updatedAt: false,
  fields: [
    {
      interface: 'string',
      type: 'string',
      name: 'title',
      title: '字段名称',
      required: true,
    },
    {
      interface: 'string',
      type: 'randomString',
      name: 'name',
      title: '标识',
      required: true,
      createOnly: true,
      randomString: {
        length: 5,
        template: 'f_%r',
        characters: 'abcdefghijklmnopqrstuvwxyz0123456789',
      },
      developerMode: true,
    },
    ...getInterfaceFields(),
    {
      interface: 'string',
      type: 'string',
      name: 'type',
      title: '数据类型',
      developerMode: true,
    },
    {
      interface: 'number',
      type: 'integer',
      name: 'parent_id',
      title: '所属分组',
    },
    {
      interface: 'linkTo',
      multiple: false,
      type: 'belongsTo',
      name: 'parent',
      title: '所属分组',
      target: 'fields',
      foreignKey: 'parent_id',
      targetKey: 'id',
      component: {
        type: 'drawerSelect',
      },
    },
    {
      interface: 'textarea',
      type: 'virtual',
      name: 'component.tooltip',
      title: '提示信息',
    },
    {
      interface: 'boolean',
      type: 'boolean',
      name: 'required',
      title: '必填项',
      defaultValue: false,
    },
    {
      interface: 'linkTo',
      type: 'belongsTo',
      name: 'collection',
      title: '所属数据表',
      target: 'collections',
      targetKey: 'name',
      labelField: 'title',
    },
    {
      interface: 'boolean',
      type: 'boolean',
      name: 'developerMode',
      title: '开发者模式',
      developerMode: true,
      defaultValue: false,
    },
    {
      interface: 'json',
      type: 'json',
      name: 'options',
      title: '配置信息',
      defaultValue: {},
      developerMode: true,
    },
  ],
} as TableOptions;
