import { TableOptions } from '@nocobase/database';

export default {
  name: 'users',
  title: '用户',
  // developerMode: true,
  // internal: true,
  createdBy: false,
  updatedBy: false,
  fields: [
    {
      interface: 'string',
      type: 'string',
      name: 'username',
      title: '用户名',
      unique: true,
      required: true,
      createOnly: true,
      component: {
        type: 'string',
        showInTable: true,
        showInDetail: true,
        showInForm: true,
      },
    },
    {
      interface: 'email',
      type: 'string',
      name: 'email',
      title: '邮箱',
      unique: true,
      required: true,
      createOnly: true,
      component: {
        type: 'string',
        showInTable: true,
        showInDetail: true,
        showInForm: true,
      },
    },
    {
      interface: 'string',
      type: 'string',
      name: 'nickname',
      title: '昵称',
      required: true,
      component: {
        type: 'string',
        showInTable: true,
        showInDetail: true,
        showInForm: true,
      },
    },
    {
      interface: 'phone',
      type: 'string',
      name: 'phone',
      unique: true,
      title: '手机号',
      component: {
        type: 'string',
        showInTable: true,
        showInDetail: true,
        showInForm: true,
      },
    },
    {
      interface: 'password',
      type: 'password',
      name: 'password',
      title: '密码',
      hidden: true,
      component: {
        type: 'password',
        showInForm: true,
      },
    },
    {
      interface: 'string',
      type: 'string',
      name: 'token',
      title: 'Token',
      unique: true,
      hidden: true,
      filterable: false,
      developerMode: true,
    },
    {
      interface: 'string',
      type: 'string',
      name: 'reset_token',
      title: 'Reset Token',
      unique: true,
      hidden: true,
      filterable: false,
      developerMode: true,
    },
  ],
  actions: [
    {
      type: 'list',
      name: 'list',
      title: '查看',
    },
    {
      type: 'destroy',
      name: 'destroy',
      title: '删除',
    },
    {
      type: 'create',
      name: 'create',
      title: '新增',
      viewName: 'form',
    },
    {
      type: 'update',
      name: 'update',
      title: '编辑',
      viewName: 'form',
    },
  ],
  views_v2: [
    {
      developerMode: true,
      type: 'table',
      name: 'table',
      title: '全部数据',
      labelField: 'nickname',
      actions: [
        {
          name: 'create',
          type: 'create',
          title: '新增',
          viewName: 'form',
        },
        {
          name: 'destroy',
          type: 'destroy',
          title: '删除',
        },
      ],
      fields: ['email', 'nickname', 'phone', 'roles'],
      detailsOpenMode: 'drawer', // window
      details: ['form'],
      sort: ['id'],
    },
    {
      developerMode: true,
      type: 'descriptions',
      name: 'descriptions',
      title: '详情',
      fields: ['email', 'nickname', 'phone', 'roles'],
      actions: [
        {
          name: 'update',
          type: 'update',
          title: '编辑',
          viewName: 'form',
        },
      ],
    },
    {
      developerMode: true,
      type: 'form',
      name: 'form',
      title: '表单',
      fields: ['email', 'nickname', 'phone', 'password', 'roles'],
    },
  ],
} as TableOptions;
